@@ -10,10 +10,10 @@ using System.Text;
 using System.Collections;
 using System.Runtime.InteropServices;
 using System.Text.RegularExpressions;
+using System.Web;
 using System.Web.Mail;
 
 using FSpot;
-using FSpot.Mail;
 using LibGPhoto2;
 
 public class MainWindow {
@@ -77,6 +77,10 @@ public class MainWindow {
 	[Glade.Widget] MenuItem zoom_in;
 	[Glade.Widget] MenuItem zoom_out;
 
+	[Glade.Widget] RadioMenuItem month;
+	[Glade.Widget] RadioMenuItem directory;
+	[Glade.Widget] CheckMenuItem reverse_order;
+
 	// Find
 	[Glade.Widget] MenuItem find_tag;
 	[Glade.Widget] CheckMenuItem find_untagged;
@@ -237,15 +241,26 @@ public class MainWindow {
 				  DragAction.Copy | DragAction.Move ); 
 
 		tag_selection_widget.ButtonPressEvent += HandleTagSelectionButtonPressEvent;
-
+		tag_selection_widget.PopupMenu += HandleTagSelectionPopupMenu;
+		
 		info_box = new InfoBox ();
 		info_box.VersionIdChanged += HandleInfoBoxVersionIdChange;
 		left_vbox.PackStart (info_box, false, true, 0);
+		
+		try {
+			query = new FSpot.PhotoQuery (db.Photos);
+		} catch (System.Exception e) {
+			//FIXME assume any exception here is due to a corrupt db and handle that.
+			RestoreDb (e);
+			query = new FSpot.PhotoQuery (db.Photos);
+		}
 
-		query = new FSpot.PhotoQuery (db.Photos);
 		query.Changed += HandleQueryChanged;
 
 		db.Photos.ItemsChanged += HandleDbItemsChanged;
+		db.Tags.ItemsChanged += HandleTagsChanged;
+		db.Tags.ItemsAdded += HandleTagsChanged;
+		db.Tags.ItemsRemoved += HandleTagsChanged;
 #if SHOW_CALENDAR
 		FSpot.SimpleCalendar cal = new FSpot.SimpleCalendar (query);
 		cal.DaySelected += HandleCalendarDaySelected;
@@ -253,9 +268,8 @@ public class MainWindow {
 #endif
 
 		group_selector = new FSpot.GroupSelector ();
-		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
+		group_selector.Adaptor = new FSpot.TimeAdaptor (query);
 
-		group_selector.Adaptor  = adaptor;
 		group_selector.ShowAll ();
 		
 		if (zoom_scale != null) {
@@ -288,6 +302,7 @@ public class MainWindow {
 		icon_view.DragDataGet += HandleIconViewDragDataGet;
 
 		TagMenu menu = new TagMenu (attach_tag, db.Tags);
+		menu.NewTagHandler += delegate { HandleCreateTagAndAttach (main_window, null); };
 		menu.TagSelected += HandleAttachTagMenuSelected;
 
 		near_image.SetFromStock ("f-spot-stock_near", IconSize.SmallToolbar);
@@ -309,7 +324,7 @@ public class MainWindow {
 		icon_view.DragDataReceived += HandleIconViewDragDataReceived;
 		icon_view.KeyPressEvent += HandleIconViewKeyPressEvent;
 
-		photo_view = new PhotoView (query, db.Photos);
+		photo_view = new PhotoView (query);
 		photo_box.Add (photo_view);
 		photo_view.ButtonPressEvent += HandlePhotoViewButtonPressEvent;
 		photo_view.KeyPressEvent += HandlePhotoViewKeyPressEvent;
@@ -333,7 +348,10 @@ public class MainWindow {
 		photo_view.DragDataReceived += HandlePhotoViewDragDataReceived;
 
 		view_notebook.SwitchPage += HandleViewNotebookSwitchPage;
-		adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.Changed += HandleAdaptorChanged;
+		LoadPreference (Preferences.GROUP_ADAPTOR);
+		LoadPreference (Preferences.GROUP_ADAPTOR_ORDER_ASC);
 
 		this.selection = new MainSelection (this);
 		this.selection.Changed += HandleSelectionChanged;
@@ -437,14 +455,24 @@ public class MainWindow {
 			Photo p = item as Photo;
 			if (p == null)
 				continue;
-			
-			if (write_metadata)
-				p.WriteMetadataToImage ();
-			
+
+			if (write_metadata) {
+				try {
+					p.WriteMetadataToImage ();
+				} catch (System.Exception e) {
+					Console.WriteLine ("Error syncing metadata to file\n{0}", e);
+				}
+			}
 		}
 		
 		if (args is TimeChangedEventArgs)
-			InvalidateViews ();
+			query.RequestReload ();
+	}
+
+	private void HandleTagsChanged (object sender, DbItemEventArgs args)
+	{
+		icon_view.QueueDraw ();
+		UpdateTagEntryFromSelection ();
 	}
 
 	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
@@ -464,7 +492,7 @@ public class MainWindow {
 	public int [] SelectedIds () {
 		int [] ids = new int [0];
 
-		if (fsview != null)
+		if (fsview != null && fsview.View.Item.IsValid)
 			ids = new int [] { fsview.View.Item.Index };
 		else {
 			switch (view_mode) {
@@ -615,6 +643,7 @@ public class MainWindow {
 	{
 		UpdateMenus ();
 		UpdateTagEntryFromSelection ();
+		info_box.Update ();
 	}
 
 
@@ -654,9 +683,9 @@ public class MainWindow {
 	// Commands
 	//
 
-	private void RotateSelectedPictures (RotateDirection direction)
+	private void RotateSelectedPictures (Gtk.Window parent, RotateDirection direction)
 	{
-		RotateCommand command = new RotateCommand (main_window);
+		RotateCommand command = new RotateCommand (parent);
 		
 		int [] selected_ids = SelectedIds ();
 		if (command.Execute (direction, SelectedPhotos (selected_ids))) {
@@ -695,21 +724,41 @@ public class MainWindow {
 		}
 	}
 
-	[GLib.ConnectBefore]
 	void HandleTagSelectionButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
 		if (args.Event.Button == 3)
 		{
 			TagPopup popup = new TagPopup ();
 			popup.Activate (args.Event, tag_selection_widget.TagAtPosition ((int)args.Event.X, (int)args.Event.Y),
-			tag_selection_widget.TagHighlight ());
+			tag_selection_widget.TagHighlight);
 			args.RetVal = true;
 		}
 	}
 
+	void HandleTagSelectionPopupMenu (object sender, PopupMenuArgs args)
+	{
+		TagPopup popup = new TagPopup ();
+		popup.Activate (null, null, tag_selection_widget.TagHighlight);
+		args.RetVal = true;
+	}
+
+	void RestoreDb (System.Exception e)
+	{
+		string backup = db.Repair ();
+		string short_msg = Mono.Posix.Catalog.GetString ("Error loading database.");
+		string long_msg = Mono.Posix.Catalog.GetString ("F-Spot encountered an error while loading the photo database. " + 
+								"The old database has be moved to {0} and a new database has been created.");
+
+		HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+							    MessageType.Error, ButtonsType.Ok, 
+							    short_msg, String.Format (long_msg, backup));
+		md.Run ();
+		md.Destroy ();
+	}
+
 	void HandleTagSelectionDragBegin (object sender, DragBeginArgs args)
 	{
-		Tag [] tags = tag_selection_widget.TagHighlight ();
+		Tag [] tags = tag_selection_widget.TagHighlight;
 		int len = tags.Length;
 		int size = 32;
 		int csize = size/2 + len * size / 2 + 2;
@@ -794,7 +843,7 @@ public class MainWindow {
 				
 				// FIXME - at this point we should import the photo, and then continue
 				if (photo == null)
-					return;
+					continue;
 				
 				// FIXME this should really follow the AddTagsExtended path too
 				photo.AddTag (tags);
@@ -803,7 +852,7 @@ public class MainWindow {
 			InvalidateViews ();
 			break;
 		case (uint)TargetType.TagList:
-			Tag child = tag_selection_widget.TagHighlight ()[0];
+			Tag child = tag_selection_widget.TagHighlight [0];
 			Tag parent = tags[0];
 
 			// FIXME with this reparenting via dnd, you cannot move a tag to root.
@@ -863,6 +912,11 @@ public class MainWindow {
 		JumpTo (index);
 	}
 
+	void HandleAdaptorChanged (FSpot.GroupAdaptor sender)
+	{
+		UpdateGlass ();
+	}
+
 	/*
 	 * Keep the glass temporal slider in sync with the user's scrolling in the icon_view
 	 */
@@ -873,7 +927,7 @@ public class MainWindow {
 			return;
 
 		int cell_num = icon_view.TopLeftVisibleCell();
-		if (cell_num == -1 || cell_num == lastTopLeftCell)
+		if (cell_num == -1 /*|| cell_num == lastTopLeftCell*/)
 			return;
 
 		lastTopLeftCell = cell_num;
@@ -899,7 +953,7 @@ public class MainWindow {
 
 	void HandleIconViewReady (object sender, EventArgs args)
 	{
-		LoadPreference (Preferences.ICON_VIEW_POSITION);
+		LoadPreference (Preferences.GLASS_POSITION);
 
 		// We only want to set the position the first time
 		// the icon_view is ready (eg on startup)
@@ -1048,9 +1102,9 @@ public class MainWindow {
 			//Console.WriteLine ("Drop cell = {0} ({1},{2})", item, args.X, args.Y);
 			if (item >= 0) {
 				if (icon_view.Selection.Contains (item))
-					AttachTags (tag_selection_widget.TagHighlight (), SelectedIds());
+					AttachTags (tag_selection_widget.TagHighlight, SelectedIds());
 				else 
-					AttachTags (tag_selection_widget.TagHighlight (), new int [] {item});
+					AttachTags (tag_selection_widget.TagHighlight, new int [] {item});
 			}
 			break;
 		case (uint)TargetType.UriList:
@@ -1375,9 +1429,8 @@ public class MainWindow {
 		StringBuilder url = new StringBuilder ("mailto:?subject=my%20photos");
 
 		foreach (Photo p in SelectedPhotos ()) {
-			url.Append ("&attach=" + p.DefaultVersionPath);
+			url.Append ("&attach=" + UriList.EscapeString (p.DefaultVersionUri.LocalPath, true, true, true));
 		}
-
 		GnomeUtil.UrlShow (main_window, url.ToString ());
 	}
 
@@ -1427,26 +1480,68 @@ public class MainWindow {
 
                 new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
-			   Mono.Posix.Catalog.GetString ("Copyright 2003-2006 Novell Inc."),
+			   Mono.Posix.Catalog.GetString ("Copyright \x00a9 2003-2006 Novell Inc."),
                            null, authors, new string [0], translators, null).Show();
 	}
 
-	void HandleArrangeByTime (object sender, EventArgs args)
+	public void HandleArrangeByTime (object sender, EventArgs args)
 	{
+		if (group_selector.Adaptor is TimeAdaptor)
+			return;
+
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
-		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
-		group_selector.Adaptor = adaptor;
+		group_selector.Adaptor.Changed -= HandleAdaptorChanged;
+		group_selector.Adaptor = new FSpot.TimeAdaptor (query);
+
 		group_selector.Mode = FSpot.GroupSelector.RangeType.Min;
-		adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.Changed += HandleAdaptorChanged;
+
+		if (sender != month)
+			month.Active = true;
+
+		//update the selection in the Timeline
+		if (query.Range != null)
+			group_selector.SetLimitsToDates(query.Range.Start, query.Range.End);
 	}
 
-	void HandleArrangeByDirectory (object sender, EventArgs args)
+	public void HandleArrangeByDirectory (object sender, EventArgs args)
 	{
+		if (group_selector.Adaptor is DirectoryAdaptor)
+			return;
+
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
-		FSpot.GroupAdaptor adaptor = new FSpot.DirectoryAdaptor (query);		
-		group_selector.Adaptor = adaptor;
+		group_selector.Adaptor.Changed -= HandleAdaptorChanged;
+		group_selector.Adaptor = new FSpot.DirectoryAdaptor (query); 	
+
 		group_selector.Mode = FSpot.GroupSelector.RangeType.Min;
-		adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.Changed += HandleAdaptorChanged;
+
+		if (sender != directory)
+			directory.Active = true;
+	}
+	
+	public void HandleReverseOrder (object sender, EventArgs args)
+	{
+		Gtk.CheckMenuItem item = sender as Gtk.CheckMenuItem;
+
+		if (group_selector.Adaptor.OrderAscending == item.Active)
+			return;
+		
+		group_selector.Adaptor.OrderAscending = item.Active;
+		icon_view.QueueDraw ();
+
+		// FIXME this is blah...we need UIManager love here
+		if (item != reverse_order)
+			reverse_order.Active = item.Active;
+		
+		//update the selection in the timeline
+		if ( query.Range != null && group_selector.Adaptor is TimeAdaptor) {
+			group_selector.SetLimitsToDates(query.Range.Start, query.Range.End);
+			
+		}
+
 	}
 
 	// Called when the user clicks the X button	
@@ -1483,10 +1578,12 @@ public class MainWindow {
 		Preferences.Set (Preferences.SHOW_TAGS,			icon_view.DisplayTags);
 		Preferences.Set (Preferences.SHOW_DATES,		icon_view.DisplayDates);
 
+		Preferences.Set (Preferences.GROUP_ADAPTOR,		(group_selector.Adaptor is DirectoryAdaptor) ? 1 : 0);
+		Preferences.Set (Preferences.GROUP_ADAPTOR_ORDER_ASC,   group_selector.Adaptor.OrderAscending);
+		Preferences.Set (Preferences.GLASS_POSITION,		group_selector.GlassPosition);
+		
 		Preferences.Set (Preferences.SIDEBAR_POSITION,		main_hpaned.Position);
 		Preferences.Set (Preferences.ZOOM,			icon_view.Zoom);
-	
-		Preferences.Set (Preferences.ICON_VIEW_POSITION, icon_view.TopLeftVisibleCell ());
 		
 		tag_selection_widget.SaveExpandDefaults ();
 
@@ -1497,7 +1594,7 @@ public class MainWindow {
 	{
 		PhotoVersionCommands.Create cmd = new PhotoVersionCommands.Create ();
 
-		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
+		if (cmd.Execute (db.Photos, CurrentPhoto, GetToplevel (null))) {
 			query.MarkChanged (ActiveIndex ());
 		}
 	}
@@ -1506,7 +1603,7 @@ public class MainWindow {
 	{
 		PhotoVersionCommands.Delete cmd = new PhotoVersionCommands.Delete ();
 
-		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
+		if (cmd.Execute (db.Photos, CurrentPhoto, GetToplevel (null))) {
 			query.MarkChanged (ActiveIndex ());
 		}
 	}
@@ -1518,7 +1615,7 @@ public class MainWindow {
 	        long length = 0;
 
 		foreach (Photo p in photos) {
-			System.IO.FileInfo fi = new System.IO.FileInfo (p.DefaultVersionPath);
+			System.IO.FileInfo fi = new System.IO.FileInfo (p.DefaultVersionUri.LocalPath);
 
 			length += fi.Length;
 		}
@@ -1534,23 +1631,34 @@ public class MainWindow {
 			query.MarkChanged (ActiveIndex ());
 		}
 	}
-
-	public void HandleCreateNewTagCommand (object sender, EventArgs args)
+	
+	public void HandleCreateTagAndAttach (object sender, EventArgs args)
 	{
-		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		command.Execute (TagCommands.TagType.Tag, tag_selection_widget.TagHighlight ());
-		
+		Tag new_tag = CreateTag (sender, args);
+
+		if (new_tag != null)
+			HandleAttachTagMenuSelected (new_tag);
 	}
 
 	public void HandleCreateNewCategoryCommand (object sender, EventArgs args)
 	{
-		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		command.Execute (TagCommands.TagType.Category, tag_selection_widget.TagHighlight ());
+		Tag new_tag = CreateTag (sender, args);
+		
+		if (new_tag != null) {
+			tag_selection_widget.ScrollTo (new_tag);
+			tag_selection_widget.TagHighlight = new Tag [] {new_tag};
+		}
+	}
+
+	public Tag CreateTag (object sender, EventArgs args)
+	{
+		TagCommands.Create command = new TagCommands.Create (db.Tags, GetToplevel (sender));
+		return command.Execute (TagCommands.TagType.Category, tag_selection_widget.TagHighlight);
 	}
 
 	public void HandleAttachTagCommand (object obj, EventArgs args)
 	{
-		AttachTags (tag_selection_widget.TagHighlight (), SelectedIds ());
+		AttachTags (tag_selection_widget.TagHighlight, SelectedIds ());
 	}
 
 	void AttachTags (Tag [] tags, int [] ids) 
@@ -1564,17 +1672,14 @@ public class MainWindow {
 
 	public void HandleRemoveTagCommand (object obj, EventArgs args)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
+		Tag [] tags = this.tag_selection_widget.TagHighlight;
 
 		db.BeginTransaction ();
 		foreach (int num in SelectedIds ()) {
 			Photo p = query.Photos [num];
 
 			p.RemoveTag (tags);
-			/*
-			if (write_metadata)
-				p.WriteMetadataToImage ();
-			*/
+
 			query.Commit (num);
 		}
 		db.CommitTransaction ();
@@ -1582,7 +1687,7 @@ public class MainWindow {
 
 	public void HandleEditSelectedTag (object sender, EventArgs ea)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
+		Tag [] tags = this.tag_selection_widget.TagHighlight;
 		if (tags.Length != 1)
 			return;
 
@@ -1596,13 +1701,11 @@ public class MainWindow {
 		
 		TagCommands.Edit command = new TagCommands.Edit (db, main_window);
 		command.Execute (tag);
-		if (view_mode == ModeType.IconView)
-			icon_view.QueueDraw ();
 	}
 
 	public void HandleMergeTagsCommand (object obj, EventArgs args)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
+		Tag [] tags = this.tag_selection_widget.TagHighlight;
 		if (tags.Length < 2)
 			return;
 		
@@ -1636,7 +1739,7 @@ public class MainWindow {
 
 		string msg = Mono.Posix.Catalog.GetString("This operation will merge the selected tags and any sub-tags into a single tag.");
 
-		string ok_caption = Mono.Posix.Catalog.GetString ("_Merge tags");
+		string ok_caption = Mono.Posix.Catalog.GetString ("_Merge Tags");
 		
 		if (ResponseType.Ok != HigMessageDialog.RunHigConfirmation(main_window, 
 									   DialogFlags.DestroyWithParent, 
@@ -1665,21 +1768,19 @@ public class MainWindow {
 		db.Photos.Remove (removetags);
 		db.CommitTransaction ();
 
-		UpdateTagEntryFromSelection ();
-
-		icon_view.QueueDraw ();
-
 		HandleEditSelectedTagWithTag (survivor);
 	}
 
 	void HandleAdjustTime (object sender, EventArgs args)
 	{
-		new TimeDialog (db, new PhotoList (Selection.Items));
+		PhotoList list = new PhotoList (Selection.Items);
+		list.Sort (new Photo.CompareDateName ());
+		new TimeDialog (db, list);
 	}
 
 	void HandleAdjustColor (object sender, EventArgs args)
 	{
-		if (ActiveIndex () > 0) {
+		if (ActiveIndex () >= 0) {
 			SetViewMode (ModeType.PhotoView);
 			new FSpot.ColorDialog (photo_view.View);
 		}
@@ -1719,15 +1820,15 @@ public class MainWindow {
 
 		if (response == Gtk.ResponseType.Ok) {
 			foreach (int id in SelectedIds ()) {
-				Gdk.Pixbuf orig = FSpot.PhotoLoader.Load (query, id);
-				Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
-			
 				Photo photo = query.Photos [id];
-				
-				bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
-
 				try {
+					Gdk.Pixbuf orig = FSpot.PhotoLoader.Load (query, id);
+					Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
+					
+					bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
+
 					photo.SaveVersion (final, create_version);
+					query.Commit (id);
 				} catch (System.Exception e) {
 					string msg = Mono.Posix.Catalog.GetString ("Error saving sharpened photo");
 					string desc = String.Format (Mono.Posix.Catalog.GetString ("Received exception \"{0}\". Unable to save photo {1}"),
@@ -1915,11 +2016,21 @@ public class MainWindow {
 		zoom_scale.ValueChanged += HandleZoomScaleValueChanged;
 	}
 
+	void HandleZoomOut (object sender, ButtonPressEventArgs args)
+	{
+		ZoomOut ();
+	}
+	
 	void HandleZoomOut (object sender, EventArgs args)
 	{
 		ZoomOut ();
 	}
 	
+	void HandleZoomIn (object sender, ButtonPressEventArgs args)
+	{
+		ZoomIn ();
+	}
+	
 	void HandleZoomIn (object sender, EventArgs args)
 	{
 		ZoomIn ();
@@ -1974,7 +2085,22 @@ public class MainWindow {
 			main_window, DialogFlags.DestroyWithParent, MessageType.Error,
 			error, msg, ok_caption);
 	}
-	
+
+	public Gtk.Window GetToplevel (object sender)
+	{
+		Widget wsender = sender as Widget;
+		Gtk.Window toplevel = null;
+
+		if (wsender != null)
+			toplevel = (Gtk.Window) wsender.Toplevel;
+		else if (fsview != null)
+			toplevel = fsview;
+		else 
+			toplevel = main_window;
+
+		return toplevel;
+	}
+
 	public void HandleDeleteCommand (object sender, EventArgs args)
 	{
    		Photo[] photos = SelectedPhotos();
@@ -1986,15 +2112,21 @@ public class MainWindow {
 								 "This deletes all versions of the selected photos from your drive.", 
 								 photos.Length);
 		string ok_caption = Mono.Posix.Catalog.GetPluralString ("_Delete photo", "_Delete photos", photos.Length);
-		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, MessageType.Warning, header, msg, ok_caption)) {                              
+		
+
+
+		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(GetToplevel (sender), 
+									   DialogFlags.DestroyWithParent, 
+									   MessageType.Warning, 
+									   header, msg, ok_caption)) {                              
 			
 			foreach (Photo photo in photos) {
 				foreach (uint id in photo.VersionIds) {
-					Console.WriteLine (" path == {0}", photo.GetVersionPath (id));
+					Console.WriteLine (" path == {0}", photo.VersionUri (id).LocalPath);
 					try {
 						photo.DeleteVersion (id, true);
 					} catch (Exception e) {
-						DeleteException (e, photo.GetVersionPath (id));
+						DeleteException (e, photo.VersionUri (id).ToString ());
 					}
 				}
 			}
@@ -2014,7 +2146,7 @@ public class MainWindow {
 		header = String.Format (header, photos.Length);
 		string msg = Mono.Posix.Catalog.GetString("If you remove photos from the F-Spot catalog all tag information will be lost. The photos remain on your computer and can be imported into F-Spot again.");
 		string ok_caption = Mono.Posix.Catalog.GetString("_Remove from Catalog");
-		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, 
+		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(GetToplevel (sender), DialogFlags.DestroyWithParent, 
 									   MessageType.Warning, header, msg, ok_caption)) {                              
 			db.Photos.Remove (photos);
 			UpdateQuery ();
@@ -2062,7 +2194,7 @@ public class MainWindow {
 
 	public void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
+		Tag [] tags = this.tag_selection_widget.TagHighlight;
 
 		System.Array.Sort (tags, new TagRemoveComparer ());
 
@@ -2101,7 +2233,6 @@ public class MainWindow {
 				md.Destroy ();
 			}
 		}
-		icon_view.QueueDraw ();
 	}
 
 	void HandleUpdateThumbnailCommand (object sende, EventArgs args)
@@ -2117,12 +2248,24 @@ public class MainWindow {
 
 	public void HandleRotate90Command (object sender, EventArgs args)
 	{
-		RotateSelectedPictures (RotateDirection.Clockwise);
+        // Don't steal characters from any text entries
+        if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
+            Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
+            return;
+        }
+
+		RotateSelectedPictures (GetToplevel (sender), RotateDirection.Clockwise);
 	}
 
 	public void HandleRotate270Command (object sender, EventArgs args)
 	{
-		RotateSelectedPictures (RotateDirection.Counterclockwise);
+        // Don't steal characters from any text entries
+        if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
+            Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
+            return;
+        }
+
+		RotateSelectedPictures (GetToplevel (sender), RotateDirection.Counterclockwise);
 	}
 
 	public void HandleCopyLocation (object sender, EventArgs args)
@@ -2140,7 +2283,7 @@ public class MainWindow {
 			if (i++ > 0)
 				paths.Append (" ");
 
-			paths.Append (System.IO.Path.GetFullPath (p.DefaultVersionPath));
+			paths.Append (System.IO.Path.GetFullPath (p.DefaultVersionUri.LocalPath));
 		}
 		
 		String data = paths.ToString ();
@@ -2160,16 +2303,22 @@ public class MainWindow {
 		client.Set ("/desktop/gnome/background/primary_color", "#000000");
 		client.Set ("/desktop/gnome/background/picture_options", "scaled");
 		client.Set ("/desktop/gnome/background/picture_opacity", 100);
-		client.Set ("/desktop/gnome/background/picture_filename", current.DefaultVersionPath);
+		client.Set ("/desktop/gnome/background/picture_filename", current.DefaultVersionUri.LocalPath);
 		client.Set ("/desktop/gnome/background/draw_background", true);
 	}
 
 	void HandleSetDateRange (object sender, EventArgs args) {
 		DateCommands.Set set_command = new DateCommands.Set (query, main_window);
 		set_command.Execute ();
+		//update the TimeLine
+		if (group_selector.Adaptor is TimeAdaptor && query.Range != null) 
+			group_selector.SetLimitsToDates(query.Range.Start, query.Range.End);
 	}
 
-	void HandleClearDateRange (object sender, EventArgs args) {
+	public void HandleClearDateRange (object sender, EventArgs args) {
+		if (group_selector.Adaptor is FSpot.TimeAdaptor) {
+			group_selector.ResetLimits();
+		}
 		query.Range = null;
 	}
 	
@@ -2251,22 +2400,38 @@ public class MainWindow {
 				//display_dates_menu_item.Toggle ();
 			break;
 		
+		case Preferences.GROUP_ADAPTOR:
+			if ((int) val == 1)
+				directory.Active = true;
+			break;
+
+		case Preferences.GROUP_ADAPTOR_ORDER_ASC:
+			if (reverse_order.Active != (bool) val)
+				reverse_order.Active = (bool) val;
+			//group_selector.Adaptor.OrderAscending = (bool) val;
+			break;
+
+		case Preferences.GLASS_POSITION:
+			if (query.Count > 0) {
+				// If the database has changed since this pref was saved, this could cause
+				// an exception to be thrown.
+				try {
+					IBrowsableItem photo = group_selector.Adaptor.PhotoFromIndex ((int) val);
+					
+					if (photo != null)
+						JumpTo (query.IndexOf (photo));
+				} catch (Exception e) {}
+			}
+			break;
 		case Preferences.SIDEBAR_POSITION:
 			if (main_hpaned.Position != (int) val)
 				main_hpaned.Position = (int) val;
 			break;
-		
+
 		case Preferences.ZOOM:
 			icon_view.Zoom = (double) val;
 			break;
 		
-		case Preferences.ICON_VIEW_POSITION:
-			if (icon_view.TopLeftVisibleCell () != (int) val) {
-				icon_view.FocusCell = (int) val;
-				photo_view.Item.Index = (int) val;
-				icon_view.ScrollTo ((int) val, false);
-			}
-			break;
 		case Preferences.METADATA_EMBED_IN_IMAGE:
 			write_metadata = (bool) val;
 			break;
@@ -2404,40 +2569,68 @@ public class MainWindow {
 		}
 	}
 
-	public void HandleOpenWith (Gnome.Vfs.MimeApplication mime_application)
+	public void HandleOpenWith (object sender, Gnome.Vfs.MimeApplication mime_application)
 	{
 		Photo[] selected = SelectedPhotos ();
 
 		if (selected == null || selected.Length < 1)
 			return;
-		
+
 		string header = Catalog.GetPluralString ("Create New Version?", "Create New Versions?", selected.Length); 
-		
 		string msg = String.Format (Catalog.GetPluralString (
 				"Before launching {1}, should F-Spot create a new version of the selected photo to preserve the original?",
 				"Before launching {1}, should F-Spot create new versions of the selected photos to preserve the originals?", selected.Length),
 				selected.Length, mime_application.Name);
 
-		// FIXME add a cancel button?
-		Gtk.ResponseType response = HigMessageDialog.RunHigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
-									   MessageType.Question, Gtk.ButtonsType.YesNo, 
-									   header, msg);
+		// FIXME add cancel button? add help button?
+		HigMessageDialog hmd = new HigMessageDialog(GetToplevel (sender), DialogFlags.DestroyWithParent, 
+							    MessageType.Question, Gtk.ButtonsType.None,
+							    header, msg);
+
+		hmd.AddButton (Gtk.Stock.No, Gtk.ResponseType.No, false);
+		//hmd.AddButton (Gtk.Stock.Cancel, Gtk.ResponseType.Cancel, false);
+		hmd.AddButton (Gtk.Stock.Yes, Gtk.ResponseType.Yes, true);
+
+		Gtk.ResponseType response = Gtk.ResponseType.Cancel;
+
+		try {
+			response = (Gtk.ResponseType) hmd.Run();
+		} finally {
+			hmd.Destroy ();
+		}
+
+		if (response == Gtk.ResponseType.Cancel)
+			return;
 
 		bool create_new_versions = (response == Gtk.ResponseType.Yes);
 
+		ArrayList errors = new ArrayList ();
 		GLib.List uri_list = new GLib.List (typeof (string));
 		foreach (Photo photo in selected) {
-			if (create_new_versions) {
-				// exception handling? Out of space, blah, blah
-				// FIXME give the new version a better name, based on the app's name that we're opening it with
-				uint version = photo.CreateNamedVersion (mime_application.Name, photo.DefaultVersionId, true);
-				photo.DefaultVersionId = version;
-				UpdateForVersionIdChange (version);
+			try {
+				if (create_new_versions) {
+					uint version = photo.CreateNamedVersion (mime_application.Name, photo.DefaultVersionId, true);
+					photo.DefaultVersionId = version;
+				}
+				query.MarkChanged (query.IndexOf (photo));
+			} catch (Exception e) {
+				errors.Add (new EditException (photo, e));
 			}
 
-			uri_list.Append (photo.DefaultVersionPath);
+			uri_list.Append (photo.DefaultVersionUri.ToString ());
 		}
-		
+
+		// FIXME need to clean up the error dialog here.
+		if (errors.Count > 0) {
+			Dialog md = new EditExceptionDialog (GetToplevel (sender), errors.ToArray (typeof (EditException)) as EditException []);
+			md.Run ();
+			md.Destroy ();
+		}
+
+		if (create_new_versions) {
+			db.Photos.Commit (selected, new DbItemEventArgs (selected));
+		}
+
 		mime_application.Launch (uri_list);
 	}
 
@@ -2505,15 +2698,14 @@ public class MainWindow {
 			return;
 #endif
 		
-		if (tag_entry.Text.Length > 0)
-			tag_entry.Text += ", ";
-
 #if ALLOW_TAG_TYPING_WITHOUT_HOTKEY
 		tag_entry.Text += c;
 #endif
 
 		tagbar.Show ();
 		UpdateTagEntryFromSelection ();
+		if (tag_entry.Text.Length > 0)
+			tag_entry.Text += ", ";
 		tag_entry.GrabFocus ();
 		tag_entry.SelectRegion (-1, -1);
 	}
@@ -2538,7 +2730,7 @@ public class MainWindow {
 
 	       // Add any new tags to the selected photos
 	       Category default_category = null;
-	       Tag [] selection = tag_selection_widget.TagHighlight ();
+	       Tag [] selection = tag_selection_widget.TagHighlight;
 	       if (selection.Length > 0) {
 		       if (selection [0] is Category)
 			       default_category = (Category) selection [0];
@@ -2583,7 +2775,6 @@ public class MainWindow {
 	       }
 
 	       if (view_mode == ModeType.IconView) {
-		       icon_view.QueueDraw ();
 		       icon_view.GrabFocus ();
 	       } else {
 		       photo_view.QueueDraw ();
@@ -2720,7 +2911,7 @@ public class MainWindow {
 		ArrayList mimes = new ArrayList ();
 
 		foreach (Photo p in SelectedPhotos ()) {
-			string mime = Gnome.Vfs.MimeType.GetMimeTypeForUri (p.DefaultVersionPath);
+			string mime = Gnome.Vfs.MimeType.GetMimeTypeForUri (p.DefaultVersionUri.ToString ());
 
 			if (! mimes.Contains (mime))
 				mimes.Add (mime);
